% $Id: exit.Rd,v 1.5 2003/12/09 20:57:16 warnes Exp $
\name{exit}
\alias{exit}
\title{Exit from a child process}
\description{
  Exit from a child process.
}
\usage{
exit(status = 0)
}
\arguments{
  \item{status}{Integer status flag. Use 0 for a normal exit.}
}
\details{
  This function is a shallow warpper adound the Unix "\_exit" command,
  and should be used instead of \code{quit()} to exit from a  process
  created via \code{fork}.
}
\value{
  None.
}
\note{
  The \code{fork} command automatically sets up an  \code{on.exit}
  function that calls \code{exit} before evaluating the \code{slave}
  argument, so it is usually not necessary to directly call exit().
  }
\references{"\_exit" man page}
\author{ Gregory R. Warnes \email{gregory\_r\_warnes\@groton.pfizer.com}}
\seealso{\code{\link{fork}}, \code{\link{getpid}}, \code{\link{wait}},
         \code{\link{kill}}, \code{\link{killall}} }
\examples{
waittest <- function()
{ 
  pid = fork(NULL)
  if(pid==0)
    {
      cat("Hi, I'm the child process and I need to explicitly call exit().")
      cat("\n\n")
      exit()
    }
  else
    {
      wait(pid)
      cat("Hi, I'm the main process, and I wait()ed until the child process\n")
      cat("finished before introducing myself.\n")
    }
}

waittest()
}
\keyword{programming}


\eof
% $Id: fork.Rd,v 1.9 2003/12/19 15:22:45 warnes Exp $
\name{fork}
\alias{fork}
\title{Create a new R process using the Unix 'fork' system call}
\description{
  Create a new R process using the Unix 'fork' system call.
}
\usage{
fork(slave)
}
\arguments{
  \item{slave}{Function to be executed in the new R process. This can be
  \code{NULL}, see details.}
}
\details{
  This function provides a thin wrapper around the Unix "fork" system
  call, which will create a new process which is an exact copy of the
  current R process, including open files and sockets including STDIN
  and STDOUT.
  
  The \code{child} parameter should  normally contain a function
  to be executed in the newly created process.  This function will be
  called in the new process, and \code{exit()} will be called when it
  returns to terminate the process.

  If you wish to explicitly control what happens in the child process,
  you can pass \code{child=NULL}, in which case you are responsible for
  managing all the details.

  First, the child process must call \code{exit()} to terminate instead
  of \code{quit}.  This is necessary to ensure that temporary files
  or directories created by the parent process are not removed by the
  child.

  Second, the child process should not attempt to use STDIN to obtain
  commands since it shares all files open at the time of the fork with
  the parent.  This includes STDIN, consequently, any code following
  the fork will be be obtained *competitively* by both processes.  This
  usually means that neither process will get a consistent picture of
  the following commands, since which process gets each line will be
  simply a matter of which process asked first.

  The best way to avoid the second problem is to simply pass a function
  to \code{fork} using the \code{slave} parameter.  Another way to
  accomplish this is to ensure that all code that needs to be executed
  has already been fed to the interpreter before the fork call occurs.
  The simplest mechinism to achieve this is to wrap the code containing
  the fork in a code block using curly brackets (\code{\{ ... \}}).
  This can be a top-level code block, or can be withing a loop or
  function call.,
 
  To illustrate, this code is a familiar C idiom for forking will NOT be
  interpreted properly:

\preformatted{
  pid = fork(slave=NULL)
      
  if(pid==0) { 
      
    cat("Hi from the child process"); exit() 
      
  } else { 
      
    cat("Hi from the parent process"); 
      
  }
}

  On the other hand, wrapping this code with curly brackets ensures it
  IS intepreted properly:
    
\preformatted{
  {
    pid = fork(slave=NULL) 
    if(pid==0) { 
      cat("Hi from the child process\n"); exit() 
    } else {
      cat("Hi from the parent process\n");
    } 
  } 
}

.
}
\value{
   This function returns the process ID of the child
   process to the parent process.  If \code{slave} is \code{NULL} the
   function returns 0 to the  child  process.
}
\references{'fork' man page}
\author{ Gregory R. Warnes \email{gregory\_r\_warnes\@groton.pfizer.com}}
\seealso{\code{\link{getpid}}, \code{\link{exit}}, \code{\link{wait}},
         \code{\link{kill}}, \code{\link{killall}} }
\examples{

###
# Count from 1 to 10 in a separate process
###

# define the function to do the work
testfun <- function()
  {
    cat("Counting in process", getpid(), "\n")
    for(i in 1:10)
      {
        i <<- i+1  # assign into Global environment
        cat("i=",i,"\n")
      }
    cat("Done counting in process", getpid(), "\n")
  }

# run normally, the function will change our value of i
i <- 0
testfun()
i 

# Run in a separate process, our value of i remains unchanged
i <- 0
{
  pid <- fork(testfun)
  wait(pid) # wait until the child finishes, then display its exit status
}
 

###
# Use a socket to communicate between two processes.  Information
# typed on the console, which is read by the initial process, will be send
# to the child process for display. 
###
\dontrun{
send <- function()
  {
     pid <- getpid()
     con1 <- socketConnection(Sys.info()["nodename"], port = 6011)
     i <- 1
     while(TRUE)
       {
         cat("[",pid,"] ",i,": ",sep="")
         data <- readLines(stdin(), n=1)
         writeLines(data, con=con1)
         if( length(grep("quit", data))>0 )
            break;
         i <- i+1
       }
     close(con1)
  }

recieve <- function()
  {
     pid <- getpid()
     con2 <- socketConnection(port = 6011, block=TRUE, server=TRUE)
     i <- 1
     while(TRUE)
       {
          data <- readLines(con2, n=1)
          cat("[",pid,"] ",i,": ",sep="")
          writeLines(data, stdout())
          if( length(grep("quit", data))>0 )
              break;
          i <- i+1
       }
     close(con2)
  }


pid <- fork(recieve)
send()
}
}
\keyword{programming}


\eof
% $Id: getpid.Rd,v 1.5 2003/12/09 20:57:16 warnes Exp $
\name{getpid}
\alias{getpid}
\title{Obtain the process id for the current process.}
\description{
  Obtain the process number for the current process.
}
\usage{
getpid()
}
\details{
  This function is a simple wrapper around the Unix "getpid" function call.
}
\value{
  Integer process id.
}
\references{Unix "getpid" man page}
\author{ Gregory R. Warnes \email{gregory\_r\_warnes\@groton.pfizer.com}}
\seealso{\code{\link{fork}}, \code{\link{exit}}, \code{\link{wait}},
         \code{\link{kill}}, \code{\link{killall}} }
\examples{

getpid()

\dontrun{
for(i in 1:10)
  fork( function() { cat("PID:", getpid(), "\n"); exit()} )
}
}
\keyword{programming}

\eof
% $Id: kill.Rd,v 1.7 2003/12/09 20:57:16 warnes Exp $
\name{kill}
\alias{kill}
\alias{killall}
\title{Send a signal to one or more processes.}
\description{ 
  \code{kill} sends a signal to a process.  \code{killall} sends a
  signal to all processes \code{fork}ed during the current session.
}
\usage{
kill(pid, signal = 15)
killall(signal = 15)
}
\arguments{
  \item{pid}{Process ID for the target process}
  \item{signal}{Signal number to send.  Defaults to 9 (SIGKILL)}
}
\details{
  The \code{kill} function provides a thin wrapper around the Unix
  "kill" system call, which sends a signal to the specified process.  The
  \code{killall} function sends a signal to all processes which have
  been \code{forked} during the current session.

  Refer to the local Unix man pages for details.
}
\value{
  \code{kill} returns 0 on successful completion, -1 on errors.
  \code{killall} does not return a value.
}
\references{"kill" and "waitpid"  man pages}
\author{ Gregory R. Warnes \email{gregory\_r\_warnes\@groton.pfizer.com}}
\seealso{\code{\link{getpid}}, \code{\link{exit}}, \code{\link{wait}},
         \code{\link{kill}}, \code{\link{killall}} }
\examples{

   # start a process that just sleeps for 10 seconds
    sleepy <- function() 
     {
       cat("Going to sleep..")
       Sys.sleep(10)
       cat("Woke up!")
     }
   pid <- fork( sleepy )

   # kill the sleeping process
   kill(pid)


}
\keyword{programming}


\eof
\name{signame}
\alias{signame}
\alias{sigval}
\alias{siglist}
\title{Obtain information about signals}.
\description{
  \code{signame} looks up signal information by the symbolic signal name.
  \code{sigval}  looks up signal information by the numeric value.
  \code{siglist} returns a table of signal information.
}
\usage{
signame(val)
sigval(name)
siglist()
}
\arguments{
  \item{val}{Integer signal value.}
  \item{name}{Symbolic signal name (with or without the SIG prefix).}
}
\details{
  These functions return information stored in a table constructed at
  compile time.
}
\value{
  A vector (\code{signame}, \code{sigval}) or data frame
  (\code{siglist}) containing the components:
  \item{name}{Symbolic name}
  \item{val}{Integer value}
  \item{desc}{Description}
}
\references{Unix "signal" man page.}
\author{ Gregory R. Warnes \email{gregory\_r\_warnes\@groton.pfizer.com}}
\seealso{\code{\link{getpid}}, \code{\link{exit}}, \code{\link{wait}},
         \code{\link{kill}}, \code{\link{killall}} }
\examples{

   # look up the numeric value for SIGABT:
   sigval("SIGKILL")

   # the 'SIG' component can be ommitted:
   sigval("HUP")

   # now look up based on value
   signame(9)

   # and get a complete table of signal information
   siglist()
}
\keyword{programming}

\eof
% $Id: wait.Rd,v 1.5 2003/12/09 20:57:16 warnes Exp $
\name{wait}
\alias{wait}
\title{Wait for child process(es) to stop or terminate.}
\description{Wait for child process(es) created using 'fork' command to
  stop or terminate.}
}
\usage{
wait(pid, nohang=FALSE, untraced=FALSE)
}
\arguments{
  \item{pid}{integer indicating the set of child processes  for
    which status is requested.  If missing or \code{NULL}, wait
    for all child processes.}
  \item{nohang}{Use the WNOHANG flag.}
  \item{untraced}{Use the WUNTRACED flag.}
}
\details{
  This function provides a thin wrapper around the Unix "wait" and
  "waitpid" system calls.   If \code{pid} is missing or \code{NULL}
  call, "wait" is called, otherwise "waitpid" is called.

  Refer to the local Unix man pages for details on these system calls
  and the meanings of the various flags.
  
}
\value{
  A vector of length 2 containing 
  \item{pid}{Process id of a child process}
  \item{status}{Status of the child process.}

  Refer to the local Unix man pages for details on these system calls
  and the meanings of the status indicator.
}
\references{"wait" and "waitpid"  man pages}
\author{ Gregory R. Warnes \email{gregory\_r\_warnes\@groton.pfizer.com}}
\seealso{\code{\link{fork}}, \code{\link{exit}}, \code{\link{getpid}},
         \code{\link{kill}}, \code{\link{killall}}}
\examples{
waittest <- function()
{ 
  pid = fork(NULL)
  if(pid==0)
    {
      cat("Hi, I'm the child process and I need to explicitly call exit().")
      cat("\n\n")
      exit()
    }
  else
    {
      wait(pid)
      cat("Hi, I'm the main process, and I wait()ed until the child process\n")
      cat("finished before introducing myself.\n")
    }
}

waittest()
}
\keyword{programming}

\eof
